## Exercise 1: User Mode Emulation

In this exercise you will learn about the User Mode Emulation on QEMU. This mode allows you to
run a program compiled for a different architecture than the host machine. In this case, we will run
a ARM program on an x86 machine. The program is a simple “Hello World” program that prints a message
to the console.

### 1.1 Write and compile simple hello world program form ARM

To compile the program for ARM, you need to install the ARM cross-compiler different from the one
downloaded in the prerequisites. The gcc-arm-linux-gnueabi package is a cross-compiler similar to
the gcc-arm-none-eabi package, but it is used to compile programs for the ARM architecture on a
linux machine. To install it, run the following
command:
```bash
sudo apt install gcc-arm-linux-gnueabi
```
Now, you can compile the hello world program for ARM by running the following command:
```bash
arm-linux-gnueabi-gcc -o helloarm.o -static hello.c
```
This command will generate an executable file called `helloarm.o`. The -static flag is used to generate a
statically linked binary, which means that all the libraries needed by the program are included in
the binary itself. This is useful when compiling and running a program on a machine that does not
have all the required libraries installed. In this case, we are compiling the program for ARM but to
ensure that the program runs on the host machine, we are including all the required libraries in the
binary.

Now, if you run the `helloarm.o` program on the host machine by running the following command:
```bash
./helloarm.o
```
You will see an error message `“-bash: ./helloarm.o: cannot execute binary file: Exec format error”` message printed to the console even though the program was compiled
for ARM. We can check if the executable was generated for ARM by running the following command:
```bash
file helloarm.o  
```
The output should be something like this: ` helloarm.o: ELF 32-bit LSB executable, ARM, EABI5
version 1 (SYSV), statically linked, not stripped ` We observe that the binary is a 32-bit ARM
executable.

### 1.2 Compile and run the program for the host machine
Now, let’s compile the program for the host machine. To observe the difference between using the
arm-linux-gnueabi-gcc compiler and your host machine’s compiler. To compile the program run the
following command:
```bash
gcc -o hello.o hello.c
```
Let’s check the architecture of the generated binary by running the following command:
```bash
file hello.o
```
Now you should obsereve that the output is different from the previous one, it should be something
like this ` hello.o: ELF 64-bit LSB executable, x86-64, version 1 (SYSV), dynamically linked, interpreter /lib64/ld-linux-x86-64.so.2, for GNU/Linux 3.2.0, not stripped `

Note that when compiling the program for the host machine, this time we did not use the -static
flag. This means that the binary is dynamically linked, which means that the libraries needed by the
program are not included in the binary itself, but are loaded at runtime. This is the default behavior
when compiling programs for the host machine and the statically linked binary is not needed.

To run the program on your host machine, you need to run the following command:
```bash
./hello.o
```
This command will run the hello.o program and print the `“Hello World”` message to the console. 

If you try to run the helloarm.o program on your host machine, you will notice an error message:
```bash
bash: ./helloarm.o: cannot execute binary file: Exec format error
```

### 1.3 Run the program on QEMU
To install QEMU User Mode Emulation you need to run the following command:
```bash
sudo apt install qemu-user
```

To run the program on QEMU, you need to run the following command:
```bash
qemu-arm hello.o
```
This command will start the User Mode Emulation on QEMU and run the hello.o program. You should see
the “Hello World” message printed to the console.





